#!/bin/sh

TEXTDOMAIN=sendmail-utils
TEXTDOMAINDIR=/usr/share/locale
debug=0

last=`date +%Y%m%d%H%M%S`
cakeydir="/usr/share/ssl/certs"

usage() {
  echo $"Usage : $0 [option]"
  echo $"Options:"
  echo $"  -f filename     ca cert file path. if this options set"
  echo $"                  don't make ca cert file, and signed with this file"
  echo $"  -k filename     ca key file path. if given -f option, must given this option"
  echo $"  -t dir          target directory path. if this option set"
  echo $"                  result saved in this directory"
  echo $"                  [ default : ${inst} ]"
  exit 1
}

realpath() {
  if [ -d "$1" ]; then
    chk=$1
  else
    tmpfilename=`echo $1 | sed 's!.*/!!g'`
    tmpfilename="/$tmpfilename"
    chk=`echo $1 | sed 's!/[^/]*$!!g'`
    if [ -f "${chk}" ]; then
      chk=.
    fi
  fi

  pushd $chk > /dev/null 2>&1
  tmp_check_dir=`pwd`
  popd > /dev/null 2>&1
  realpath_result="$tmp_check_dir$tmpfilename"
  export realpath_result
}

conv_color() {
  if [ -z "${1}" ]; then
    input=green
  else
    input=${1}
  fi

  case "${input}" in
    gray) MCOLS=30 ;;
    bgray) MCOLS=40 ;;
    red) MCOLS=31 ;;
    bred) MCOLS=41 ;;
    green) MCOLS=32 ;;
    bgreen) MCOLS=42 ;;
    yellow) MCOLS=33 ;;
    byellow) MCOLS=43 ;;
    blue) MCOLS=34 ;;
    bblue) MCOLS=44 ;;
    magenta) MCOLS=35 ;;
    bmagenta) MCOLS=45 ;;
    cyan) MCOLS=36 ;;
    bcyan) MCOLS=46 ;;
    white) MCOLS=37 ;;
    bwhite) MCOLS=47 ;;
    *) MCOLS=32
  esac

  return ${MCOLS}
}

start_ansi() {
  conv_color $1
  MCOL=$?

  echo -ne "[1;${MCOL}m"
}

end_ansi() {
  echo -ne "[7;0m"
}


makeCnf() {

  if [ "$1" = "ca" ]; then
    commonName=$"Common Name (eg, CA Name)"
    commonName_default=""
    organizationalUnitName_default="Cert Team"
  else
    commonName=$"Common Name (eg, FQDN)"
    commonName_default=localhost
    organizationalUnitName_default=""
  fi

  cat > sendmail.cnf <<EOT
HOME			= .
RANDFILE		= \$ENV::HOME/.rnd

oid_section		= new_oids

[ new_oids ]

[ ca ]
default_ca	= CA_default

[ CA_default ]

dir		= .
certs		= \$dir/certs
crl_dir		= \$dir/crl
database	= \$dir/index.txt
new_certs_dir	= \$dir/newcerts

certificate	= \$dir/cacert.pem
serial		= \$dir/serial
crl		= \$dir/crl.pem
private_key	= \$dir/private/cakey.pem
RANDFILE	= \$dir/private/.rand

x509_extensions	= usr_cert

default_days	= 365
default_crl_days= 30
default_md	= md5
preserve	= no

policy		= policy_match

[ policy_match ]
countryName		= match
stateOrProvinceName	= match
organizationName	= match
organizationalUnitName	= optional
commonName		= supplied
emailAddress		= optional

[ policy_anything ]
countryName		= optional
stateOrProvinceName	= optional
localityName		= optional
organizationName	= optional
organizationalUnitName	= optional
commonName		= supplied
emailAddress		= optional

[ req ]
default_bits		= 1024
default_keyfile 	= privkey.pem
distinguished_name	= req_distinguished_name
attributes		= req_attributes
x509_extensions	= v3_ca

string_mask = nombstr

[ req_distinguished_name ]
countryName			= $countryName
countryName_default		= KR
countryName_min			= 2
countryName_max			= 2

stateOrProvinceName		= $stateOrProvinceName
stateOrProvinceName_default	=

localityName			= $localityName
localityName_default		= Seoul

0.organizationName		= $organizationName
0.organizationName_default	= Your Company

organizationalUnitName		= $organizationalUnitName
organizationalUnitName_default	= $organizationalUnitName_default

commonName			= $commonName
commonName_max			= 64
commonName_default		= $commonName_default

emailAddress			= $emailAddress
emailAddress_max		= 40
emailAddress_default		= root@localhost

[ req_attributes ]
challengePassword		= A challenge password
challengePassword_min		= 4
challengePassword_max		= 20

unstructuredName		= An optional company name

[ usr_cert ]

basicConstraints=CA:FALSE

nsComment			= "OpenSSL Generated Certificate"

subjectKeyIdentifier=hash
authorityKeyIdentifier=keyid,issuer:always

[ v3_req ]

basicConstraints = CA:FALSE
keyUsage = nonRepudiation, digitalSignature, keyEncipherment

[ v3_ca ]

subjectKeyIdentifier=hash

authorityKeyIdentifier=keyid:always,issuer:always

basicConstraints = CA:true

[ crl_ext ]

authorityKeyIdentifier=keyid:always,issuer:always
EOT
}

tempDir=
targetDir=
certfile=
keyfile=
chkopt=0
step=1

inst=/usr/share/ssl/certs/sendmail
openssl=/usr/bin/openssl

countryName=$"Country Name (2 letter code)"
stateOrProvinceName=$"State or Province Name (full name)"
localityName=$"Locality Name (eg, city)"
organizationName=$"Organization Name (eg, company)"
organizationalUnitName=$"Organizational Unit Name (eg, section)"
emailAddress=$"Email Address"


echo
start_ansi blue
echo "SENDMAIL STARTTLS MAKE KEY Utility"
echo "Copyright (c) 2004 JoungKyun.Kim <http://oops.org>"
end_ansi
echo

sort_r=`getopt f:k:t: $*`
chkopt=$?
set -- $sort_r

if [ $chkopt != 0 ]; then
  usage
fi

while [ -n "$1" ]
do
  case "$1" in
    -f)
        certfile=$2
        realpath $certfile
        certfile=$realpath_result
        if [ ! -f $certfile ] ; then
          echo $"File not found $certfile"
          usage
        fi
        shift
        shift
        ;;
    -t)
        inst=$2
        realpath $inst
        inst=$realpath_result
        shift
        shift
        ;;
    -k)
        keyfile=$2
        realpath $keyfile
        keyfile=$realpath_result
        if [ ! -f $keyfile ] ; then
          echo $"File not found $keyfile"
          usage
        fi
        shift
        shift
        ;;
    --)
        shift
        break
        ;;
    *)
        usage
  esac
done

if [ $# != 0 ]; then
  usage
fi

if [ -n "$certfile" -a -z "$keyfile" ]; then
  echo "Error: Don't give -k option"
  usage
fi


# create temp directory
tmp=`mktemp -d "${TMPDIR:-/tmp}/pxboard.$$.XXXXXX"` || exit 1

pushd ${tmp} > /dev/null 2>&1
mkdir certs crl newcerts private
if [ ! -f serial ]; then
  echo '01' > serial
fi
cp /dev/null index.txt

if [ -n "${certfile}" ]; then
  if [ ! -f "${certfile}" ]; then
    popd > /dev/null 2>&1
    [ $debug -eq 0 ] && rm -rf ${tmp}
    echo $"not found ${certfile}"
    exit 1
  fi

  cp -af ${certfile} ./cacert.pem
  cp -af ${keyfile} ./private/cakey.pem

  #for file in /var/log/messages /var/adm/messages \
  #            /kernel /vmunix /vmlinuz \
  #            /etc/hosts /etc/resolv.conf; do
  #  if [ -f $file ]; then
  #    if [ ".$randfiles" = . ]; then
  #       randfiles="$file"
  #    else
  #       randfiles="${randfiles}:$file"
  #    fi
  #  fi
  #done
  #
  #if [ ".$randfiles" != . ]; then
  #  ${openssl} genrsa -rand $randfiles -out private/cakey.pem 1024
  #else
  #  ${openssl} genrsa -out private/cakey.pem 1024
  #fi
else
  echo
  start_ansi green
  echo $"STEP ${step}. Generating custom Certificate Authority (CA)"
  echo "---------------------------------------------------------------------"
  end_ansi
  echo
  echo
  sleep 1
  makeCnf ca

  # 키관리를 위하여 암호를 입력할 경우에는 이 라인을 사용해야 함
  #${openssl} req -new -x509 -keyout private/cakey.pem -out cacert.pem -days 3650 -config sendmail.cnf
  # 암호 관리가 싫으면 이 라인을 사용
  for file in /var/log/messages /var/adm/messages \
              /kernel /vmunix /vmlinuz \
              /etc/hosts /etc/resolv.conf; do
    if [ -f $file ]; then
      if [ ".$randfiles" = . ]; then
         randfiles="$file"
      else
         randfiles="${randfiles}:$file"
      fi
    fi
  done

  if [ ".$randfiles" != . ]; then
    ${openssl} genrsa -rand $randfiles -out private/cakey.pem 1024
  else
    ${openssl} genrsa -out private/cakey.pem 1024
  fi

  if [ $? -ne 0 ]; then
    popd > /dev/null 2>&1
    [ $debug -eq 0 ] && rm -rf ${tmp}
    echo
    echo $"Error: Failed to generate RSA private key" 1>&2
    exit 1
  fi

  ${openssl} req -new -x509 -key private/cakey.pem -out cacert.pem -days 3650 -config sendmail.cnf

  if [ $? -ne 0 ]; then
    popd > /dev/null 2>&1
    [ $debug -eq 0 ] && rm -rf ${tmp}
    echo $"Error: Failed to generate certificate signing request" 1>&2
    exit 1
  fi

  echo
  echo $"Result:"
  echo "---------------------------------------------------------------------"
  echo
  ${openssl} verify cacert.pem
  if [ $? -ne 0 ]; then
    popd > /dev/null 2>&1
    [ $debug -eq 0 ] && rm -rf ${tmp}
    echo
    echo $"Error: Failed to verify resulting X.509 certificate" 1>&2
    exit 1
  fi

  step=$(( $step + 1 ))
fi

echo
echo
start_ansi green
echo $"STEP ${step}. Generating custom SERVER Certificate"
echo "---------------------------------------------------------------------"
end_ansi
echo
echo
sleep 1
makeCnf
# make a new certificate
${openssl} req -nodes -new -x509 -keyout newreq.pem -out newreq.pem -days 3650 -config sendmail.cnf
if [ $? -ne 0 ]; then
  echo $"Error: Failed to generate X.509 certificate" 1>&2
  exit 1
fi

echo
echo $"Result:"
echo "---------------------------------------------------------------------"
echo
grep -A 100 "BEGIN CERTIFICATE" newreq.pem > check-tmp.pem
${openssl} verify -CAfile cacert.pem check-tmp.pem
if [ $? -ne 0 ]; then
  [ $debug -eq 0 ] && rm -f check-tmp.pem
  popd > /dev/null 2>&1
  [ $debug -eq 0 ] && rm -rf ${tmp}
  echo
  echo $"Error: Failed to verify resulting X.509 certificate" 1>&2
  exit 1
fi
[ $debug -eq 0 ] && rm -f check-tmp.pem
step=$(( $step + 1 ))


echo
echo
start_ansi green
echo $"STEP ${step}. Signication SERVER Certifacate With CA Certificate"
echo "---------------------------------------------------------------------"
end_ansi
echo
echo
sleep 1
# sign new certificate with certificate authority
${openssl} x509 -x509toreq -in newreq.pem -signkey newreq.pem -out tmp.pem
if [ $? -ne 0 ]; then
  popd > /dev/null 2>&1
  [ $debug -eq 0 ] && rm -rf ${tmp}
  echo
  echo $"Error: Convert failed x509 to req" 1>&2
  exit 1
fi
${openssl} ca -config sendmail.cnf -policy policy_anything -out newcert.pem -infiles tmp.pem
if [ $? -ne 0 ]; then
  popd > /dev/null 2>&1
  [ $debug -eq 0 ] && rm -rf ${tmp}
  echo
  echo $"Error: Failed signication" 1>&2
  exit 1
fi

echo
echo $"Result:"
echo "---------------------------------------------------------------------"
echo
${openssl} verify -CAfile cacert.pem newcert.pem
if [ $? -ne 0 ]; then
  popd > /dev/null 2>&1
  [ $debug -eq 0 ] && rm -rf ${tmp}
  echo
  echo $"Error: Failed to verify resulting X.509 certificate" 1>&2
  exit 1
fi

step=$(( $step + 1 ))

echo
echo
start_ansi green
echo $"STEP ${step}. Installation"
echo "---------------------------------------------------------------------"
end_ansi
echo
echo

if [ -d "${inst}" ]; then
  echo $"Moved ${inst} to ${inst}.old"
  mv -f ${inst} ${inst}.old
fi

mkdir -p ${inst}
echo $"CA Certificate installed to ${inst}/cacert.pem"
install -m644 cacert.pem ${inst}
echo $"Server Certificate key file install to ${inst}/key.pem"
grep -B 100  "END RSA PRIVATE KEY" newreq.pem > ${inst}/key.pem
chmod 400 ${inst}/key.pem
echo $"Client Certificate install to ${inst}/cert.pem"
install -m644 newcert.pem ${inst}/cert.pem

echo $"Backup CA Key to ${cakeydir}"
if [ -f "${cakeydir}/cakey.pem" ]; then
  mv -f ${cakeydir}/cakey.pem ${cakeydir}/cakey.pem-${last}
fi
install -m644 private/cakey.pem ${cakeydir}/cakey.pem

popd > /dev/null 2>&1
[ $debug -eq 0 ] && rm -rf ${tmp}

exit 0
